@@ -752,7 +752,7 @@ mm_bearer_properties_cmp (MMBearerProperties         *a,
         return FALSE;
     if (!cmp_ip_type (a->priv->ip_type, b->priv->ip_type, flags))
         return FALSE;
-    if (!cmp_allowed_auth (a->priv->allowed_auth, a->priv->allowed_auth, flags))
+    if (!cmp_allowed_auth (a->priv->allowed_auth, b->priv->allowed_auth, flags))
         return FALSE;
     if (!cmp_str (a->priv->user, b->priv->user, flags))
         return FALSE;
@@ -764,8 +764,10 @@ mm_bearer_properties_cmp (MMBearerProperties         *a,
         if (a->priv->allow_roaming_set != b->priv->allow_roaming_set)
             return FALSE;
     }
-    if (a->priv->rm_protocol != b->priv->rm_protocol)
-        return FALSE;
+    if (!(flags & MM_BEARER_PROPERTIES_CMP_FLAGS_NO_RM_PROTOCOL)) {
+        if (a->priv->rm_protocol != b->priv->rm_protocol)
+            return FALSE;
+    }
     return TRUE;
 }
 
