@@ -67,6 +67,7 @@ struct _MMModemClass {
 };
 
 GType mm_modem_get_type (void);
+G_DEFINE_AUTOPTR_CLEANUP_FUNC (MMModem, g_object_unref)
 
 const gchar *mm_modem_get_path (MMModem *self);
 gchar       *mm_modem_dup_path (MMModem *self);
@@ -74,6 +75,11 @@ gchar       *mm_modem_dup_path (MMModem *self);
 const gchar       *mm_modem_get_sim_path             (MMModem *self);
 gchar             *mm_modem_dup_sim_path             (MMModem *self);
 
+const gchar * const *mm_modem_get_sim_slot_paths (MMModem *self);
+gchar              **mm_modem_dup_sim_slot_paths (MMModem *self);
+
+guint              mm_modem_get_primary_sim_slot (MMModem *self);
+
 gboolean           mm_modem_peek_supported_capabilities (MMModem *self,
                                                          const MMModemCapability **capabilities,
                                                          guint *n_capabilities);
@@ -99,6 +105,14 @@ gchar             *mm_modem_dup_model                (MMModem *self);
 const gchar       *mm_modem_get_revision             (MMModem *self);
 gchar             *mm_modem_dup_revision             (MMModem *self);
 
+const gchar       *mm_modem_get_carrier_configuration          (MMModem *self);
+gchar             *mm_modem_dup_carrier_configuration          (MMModem *self);
+const gchar       *mm_modem_get_carrier_configuration_revision (MMModem *self);
+gchar             *mm_modem_dup_carrier_configuration_revision (MMModem *self);
+
+const gchar       *mm_modem_get_hardware_revision    (MMModem *self);
+gchar             *mm_modem_dup_hardware_revision    (MMModem *self);
+
 const gchar       *mm_modem_get_device_identifier    (MMModem *self);
 gchar             *mm_modem_dup_device_identifier    (MMModem *self);
 
@@ -335,6 +349,30 @@ MMSim *mm_modem_get_sim_sync   (MMModem *self,
                                 GCancellable *cancellable,
                                 GError **error);
 
+void       mm_modem_list_sim_slots        (MMModem              *self,
+                                           GCancellable         *cancellable,
+                                           GAsyncReadyCallback   callback,
+                                           gpointer              user_data);
+GPtrArray *mm_modem_list_sim_slots_finish (MMModem              *self,
+                                           GAsyncResult         *res,
+                                           GError              **error);
+GPtrArray *mm_modem_list_sim_slots_sync   (MMModem              *self,
+                                           GCancellable         *cancellable,
+                                           GError              **error);
+
+void     mm_modem_set_primary_sim_slot        (MMModem              *self,
+                                               guint                 sim_slot,
+                                               GCancellable         *cancellable,
+                                               GAsyncReadyCallback   callback,
+                                               gpointer              user_data);
+gboolean mm_modem_set_primary_sim_slot_finish (MMModem              *self,
+                                               GAsyncResult         *res,
+                                               GError              **error);
+gboolean mm_modem_set_primary_sim_slot_sync   (MMModem              *self,
+                                               guint                 sim_slot,
+                                               GCancellable         *cancellable,
+                                               GError              **error);
+
 G_END_DECLS
 
 #endif /* _MM_MODEM_H_ */
