@@ -217,20 +217,34 @@ parse_bands (guint                      bandlist,
 }
 
 static guint
-take_and_convert_from_matched_string (gchar                  *str,
-                                      MMModemCharset          charset,
-                                      MMCinterionModemFamily  modem_family)
+take_and_convert_from_matched_string (gchar                   *str,
+                                      MMModemCharset           charset,
+                                      MMCinterionModemFamily   modem_family,
+                                      GError                 **error)
 {
-    guint val = 0;
+    guint             val = 0;
+    g_autofree gchar *utf8 = NULL;
+    g_autofree gchar *taken_str = str;
 
-    if (!str)
+    if (!taken_str) {
+        g_set_error (error, MM_CORE_ERROR, MM_CORE_ERROR_INVALID_ARGS,
+                     "Couldn't convert to integer number: no input string");
         return 0;
+    }
 
-    if (modem_family == MM_CINTERION_MODEM_FAMILY_IMT)
-        str = mm_charset_take_and_convert_to_utf8 (str, charset);
+    if (modem_family == MM_CINTERION_MODEM_FAMILY_IMT) {
+        utf8 = mm_modem_charset_str_to_utf8 (taken_str, -1, charset, FALSE, error);
+        if (!utf8) {
+            g_prefix_error (error, "Couldn't convert to integer number: ");
+            return 0;
+        }
+    }
 
-    mm_get_uint_from_hex_str (str, &val);
-    g_free (str);
+    if (!mm_get_uint_from_hex_str (utf8 ? utf8 : taken_str, &val)) {
+        g_set_error (error, MM_CORE_ERROR, MM_CORE_ERROR_FAILED,
+                     "Couldn't convert to integer number: wrong hex encoding: %s", utf8 ? utf8 : taken_str);
+        return 0;
+    }
 
     return val;
 }
@@ -292,12 +306,16 @@ mm_cinterion_parse_scfg_test (const gchar                 *response,
         goto finish;
     }
 
-    r2 = g_regex_new ("\\^SCFG:\\s*\"Radio/Band/([234]G)\",\\(\"?([0-9A-Fa-fx]*)\"?-\"?([0-9A-Fa-fx]*)\"?\\)(,*\\(\"?([0-9A-Fa-fx]*)\"?-\"?([0-9A-Fa-fx]*)\"?\\))?",
+    r2 = g_regex_new ("\\^SCFG:\\s*\"Radio/Band/([234]G)\","
+                      "\\(\"?([0-9A-Fa-fx]*)\"?-\"?([0-9A-Fa-fx]*)\"?\\)"
+                      "(,*\\(\"?([0-9A-Fa-fx]*)\"?-\"?([0-9A-Fa-fx]*)\"?\\))?",
                      0, 0, NULL);
     g_assert (r2 != NULL);
+
     g_regex_match_full (r2, response, strlen (response), 0, 0, &match_info2, &inner_error);
     if (inner_error)
         goto finish;
+
     while (g_match_info_matches (match_info2)) {
         g_autofree gchar *techstr = NULL;
         guint             maxband;
@@ -306,16 +324,28 @@ mm_cinterion_parse_scfg_test (const gchar                 *response,
 
         techstr = mm_get_string_unquoted_from_match_info (match_info2, 1);
         if (g_strcmp0 (techstr, "2G") == 0) {
-            maxband = take_and_convert_from_matched_string (mm_get_string_unquoted_from_match_info (match_info2, 3), charset, modem_family);
+            maxband = take_and_convert_from_matched_string (mm_get_string_unquoted_from_match_info (match_info2, 3),
+                                                            charset, modem_family, &inner_error);
+            if (inner_error)
+                break;
             parse_bands (maxband, &bands, MM_CINTERION_RB_BLOCK_GSM, modem_family);
         } else if (g_strcmp0 (techstr, "3G") == 0) {
-            maxband = take_and_convert_from_matched_string (mm_get_string_unquoted_from_match_info (match_info2, 3), charset, modem_family);
+            maxband = take_and_convert_from_matched_string (mm_get_string_unquoted_from_match_info (match_info2, 3),
+                                                            charset, modem_family, &inner_error);
+            if (inner_error)
+                break;
             parse_bands (maxband, &bands, MM_CINTERION_RB_BLOCK_UMTS, modem_family);
         } else if (g_strcmp0 (techstr, "4G") == 0) {
-            maxband = take_and_convert_from_matched_string (mm_get_string_unquoted_from_match_info (match_info2, 3), charset, modem_family);
+            maxband = take_and_convert_from_matched_string (mm_get_string_unquoted_from_match_info (match_info2, 3),
+                                                            charset, modem_family, &inner_error);
+            if (inner_error)
+                break;
             parse_bands (maxband, &bands, MM_CINTERION_RB_BLOCK_LTE_LOW, modem_family);
             if (modem_family == MM_CINTERION_MODEM_FAMILY_DEFAULT) {
-                maxband = take_and_convert_from_matched_string (mm_get_string_unquoted_from_match_info (match_info2, 6), charset, modem_family);
+                maxband = take_and_convert_from_matched_string (mm_get_string_unquoted_from_match_info (match_info2, 6),
+                                                                charset, modem_family, &inner_error);
+                if (inner_error)
+                    break;
                 parse_bands (maxband, &bands, MM_CINTERION_RB_BLOCK_LTE_HIGH, modem_family);
             }
         } else {
@@ -407,9 +437,11 @@ mm_cinterion_parse_scfg_response (const gchar                  *response,
     if (format == MM_CINTERION_RADIO_BAND_FORMAT_SINGLE) {
         r = g_regex_new ("\\^SCFG:\\s*\"Radio/Band\",\\s*\"?([0-9a-fA-F]*)\"?", 0, 0, NULL);
         g_assert (r != NULL);
+
         g_regex_match_full (r, response, strlen (response), 0, 0, &match_info, &inner_error);
         if (inner_error)
             goto finish;
+
         if (g_match_info_matches (match_info)) {
             g_autofree gchar *currentstr = NULL;
             guint current = 0;
@@ -438,26 +470,40 @@ mm_cinterion_parse_scfg_response (const gchar                  *response,
         r = g_regex_new ("\\^SCFG:\\s*\"Radio/Band/([234]G)\",\"?([0-9A-Fa-fx]*)\"?,?\"?([0-9A-Fa-fx]*)?\"?",
                          0, 0, NULL);
         g_assert (r != NULL);
+
         g_regex_match_full (r, response, strlen (response), 0, 0, &match_info, &inner_error);
         if (inner_error)
             goto finish;
+
         while (g_match_info_matches (match_info)) {
             g_autofree gchar *techstr = NULL;
             guint current;
 
             techstr = mm_get_string_unquoted_from_match_info (match_info, 1);
-            if (g_strcmp0(techstr, "2G") == 0) {
-                current = take_and_convert_from_matched_string (mm_get_string_unquoted_from_match_info (match_info, 2), charset, modem_family);
+            if (g_strcmp0 (techstr, "2G") == 0) {
+                current = take_and_convert_from_matched_string (mm_get_string_unquoted_from_match_info (match_info, 2),
+                                                                charset, modem_family, &inner_error);
+                if (inner_error)
+                    break;
                 parse_bands (current, &bands, MM_CINTERION_RB_BLOCK_GSM, modem_family);
 
-            } else if (g_strcmp0(techstr, "3G") == 0) {
-                current = take_and_convert_from_matched_string (mm_get_string_unquoted_from_match_info (match_info, 2), charset, modem_family);
+            } else if (g_strcmp0 (techstr, "3G") == 0) {
+                current = take_and_convert_from_matched_string (mm_get_string_unquoted_from_match_info (match_info, 2),
+                                                                charset, modem_family, &inner_error);
+                if (inner_error)
+                    break;
                 parse_bands (current, &bands, MM_CINTERION_RB_BLOCK_UMTS, modem_family);
-            } else if (g_strcmp0(techstr, "4G") == 0) {
-                current = take_and_convert_from_matched_string (mm_get_string_unquoted_from_match_info (match_info, 2), charset, modem_family);
+            } else if (g_strcmp0 (techstr, "4G") == 0) {
+                current = take_and_convert_from_matched_string (mm_get_string_unquoted_from_match_info (match_info, 2),
+                                                                charset, modem_family, &inner_error);
+                if (inner_error)
+                    break;
                 parse_bands (current, &bands, MM_CINTERION_RB_BLOCK_LTE_LOW, modem_family);
                 if (modem_family == MM_CINTERION_MODEM_FAMILY_DEFAULT) {
-                    current = take_and_convert_from_matched_string (mm_get_string_unquoted_from_match_info (match_info, 3), charset, modem_family);
+                    current = take_and_convert_from_matched_string (mm_get_string_unquoted_from_match_info (match_info, 3),
+                                                                    charset, modem_family, &inner_error);
+                    if (inner_error)
+                        break;
                     parse_bands (current, &bands, MM_CINTERION_RB_BLOCK_LTE_HIGH, modem_family);
                 }
             } else {
@@ -894,19 +940,23 @@ mm_cinterion_parse_sgauth_response (const gchar          *response,
 /*****************************************************************************/
 /* ^SMONG response parser */
 
-static MMModemAccessTechnology
-get_access_technology_from_smong_gprs_status (guint    gprs_status,
-                                              GError **error)
+static gboolean
+get_access_technology_from_smong_gprs_status (guint                     gprs_status,
+                                              MMModemAccessTechnology  *out,
+                                              GError                  **error)
 {
     switch (gprs_status) {
     case 0:
-        return MM_MODEM_ACCESS_TECHNOLOGY_UNKNOWN;
+        *out = MM_MODEM_ACCESS_TECHNOLOGY_UNKNOWN;
+        return TRUE;
     case 1:
     case 2:
-        return MM_MODEM_ACCESS_TECHNOLOGY_GPRS;
+        *out = MM_MODEM_ACCESS_TECHNOLOGY_GPRS;
+        return TRUE;
     case 3:
     case 4:
-        return MM_MODEM_ACCESS_TECHNOLOGY_EDGE;
+        *out = MM_MODEM_ACCESS_TECHNOLOGY_EDGE;
+        return TRUE;
     default:
         break;
     }
@@ -917,7 +967,7 @@ get_access_technology_from_smong_gprs_status (guint    gprs_status,
                  "Couldn't get network capabilities, "
                  "unsupported GPRS status value: '%u'",
                  gprs_status);
-    return MM_MODEM_ACCESS_TECHNOLOGY_UNKNOWN;
+    return FALSE;
 }
 
 gboolean
@@ -925,9 +975,10 @@ mm_cinterion_parse_smong_response (const gchar              *response,
                                    MMModemAccessTechnology  *access_tech,
                                    GError                  **error)
 {
-    GError     *inner_error = NULL;
-    GMatchInfo *match_info = NULL;
-    GRegex     *regex;
+    guint                  value = 0;
+    GError                *inner_error = NULL;
+    g_autoptr(GMatchInfo)  match_info = NULL;
+    g_autoptr(GRegex)      regex;
 
     /* The AT^SMONG command returns a cell info table, where the second
      * column identifies the "GPRS status", which is exactly what we want.
@@ -946,26 +997,21 @@ mm_cinterion_parse_smong_response (const gchar              *response,
                          0, NULL);
     g_assert (regex);
 
-    if (g_regex_match_full (regex, response, strlen (response), 0, 0, &match_info, &inner_error)) {
-        guint value = 0;
-
-        if (!mm_get_uint_from_match_info (match_info, 2, &value))
-            inner_error = g_error_new (MM_CORE_ERROR, MM_CORE_ERROR_FAILED,
-                                       "Couldn't read 'GPRS status' field from AT^SMONG response");
-        else if (access_tech)
-            *access_tech = get_access_technology_from_smong_gprs_status (value, &inner_error);
-    }
-
-    g_match_info_free (match_info);
-    g_regex_unref (regex);
+    g_regex_match_full (regex, response, strlen (response), 0, 0, &match_info, &inner_error);
 
     if (inner_error) {
+        g_prefix_error (&inner_error, "Failed to match AT^SMONG response: ");
         g_propagate_error (error, inner_error);
         return FALSE;
     }
 
-    g_assert (access_tech != MM_MODEM_ACCESS_TECHNOLOGY_UNKNOWN);
-    return TRUE;
+    if (!g_match_info_matches (match_info) || !mm_get_uint_from_match_info (match_info, 2, &value)) {
+        g_set_error (error, MM_CORE_ERROR, MM_CORE_ERROR_FAILED,
+                     "Couldn't read 'GPRS status' field from AT^SMONG response");
+        return FALSE;
+    }
+
+    return get_access_technology_from_smong_gprs_status (value, access_tech, error);
 }
 
 /*****************************************************************************/
@@ -1468,18 +1514,36 @@ mm_cinterion_provcfg_response_to_cid (const gchar             *response,
     g_autoptr(GRegex)      r = NULL;
     g_autoptr(GMatchInfo)  match_info = NULL;
     g_autofree gchar      *mno = NULL;
+    GError                *inner_error = NULL;
 
-    r = g_regex_new ("\\^SCFG:\\s*\"MEopMode/Prov/Cfg\",\\s*\"([0-9a-zA-Z]*)\"", 0, 0, NULL);
+    r = g_regex_new ("\\^SCFG:\\s*\"MEopMode/Prov/Cfg\",\\s*\"([0-9a-zA-Z*]*)\"", 0, 0, NULL);
     g_assert (r != NULL);
 
-    if (!g_regex_match_full (r, response, strlen (response), 0, 0, &match_info, error))
+    g_regex_match_full (r, response, strlen (response), 0, 0, &match_info, &inner_error);
+
+    if (inner_error) {
+        g_prefix_error (&inner_error, "Failed to match Prov/Cfg response: ");
+        g_propagate_error (error, inner_error);
         return FALSE;
+    }
+
+    if (!g_match_info_matches (match_info)) {
+        g_set_error (error, MM_CORE_ERROR, MM_CORE_ERROR_FAILED,
+                     "Couldn't match Prov/Cfg response");
+        return FALSE;
+    }
 
     mno = mm_get_string_unquoted_from_match_info (match_info, 1);
     if (mno && modem_family == MM_CINTERION_MODEM_FAMILY_IMT) {
-        mno = mm_charset_take_and_convert_to_utf8 (mno, charset);
-        mm_obj_dbg (log_object, "current mno: %s", mno);
+        gchar *mno_utf8;
+
+        mno_utf8 = mm_modem_charset_str_to_utf8 (mno, -1, charset, FALSE, error);
+        if (!mno_utf8)
+            return FALSE;
+        g_free (mno);
+        mno = mno_utf8;
     }
+    mm_obj_dbg (log_object, "current mno: %s", mno ? mno : "none");
 
     /* for Cinterion LTE modules, some CID numbers have special meaning.
      * This is dictated by the chipset and by the MNO:
