@@ -13,6 +13,7 @@
  * Copyright (C) 2008 - 2009 Novell, Inc.
  * Copyright (C) 2009 - 2011 Red Hat, Inc.
  * Copyright (C) 2011 Google, Inc.
+ * Copyright (C) 2015 - Marco Bascetta <marco.bascetta@sadel.it>
  */
 
 #ifndef MM_BROADBAND_MODEM_H
@@ -23,6 +24,8 @@
 
 #include <ModemManager.h>
 
+#include "mm-modem-helpers.h"
+#include "mm-charsets.h"
 #include "mm-base-modem.h"
 
 #define MM_TYPE_BROADBAND_MODEM            (mm_broadband_modem_get_type ())
@@ -36,6 +39,9 @@ typedef struct _MMBroadbandModem MMBroadbandModem;
 typedef struct _MMBroadbandModemClass MMBroadbandModemClass;
 typedef struct _MMBroadbandModemPrivate MMBroadbandModemPrivate;
 
+#define MM_BROADBAND_MODEM_FLOW_CONTROL        "broadband-modem-flow-control"
+#define MM_BROADBAND_MODEM_INDICATORS_DISABLED "broadband-modem-indicators-disabled"
+
 struct _MMBroadbandModem {
     MMBaseModem parent;
     MMBroadbandModemPrivate *priv;
@@ -86,6 +92,7 @@ struct _MMBroadbandModemClass {
 };
 
 GType mm_broadband_modem_get_type (void);
+G_DEFINE_AUTOPTR_CLEANUP_FUNC (MMBroadbandModem, g_object_unref)
 
 MMBroadbandModem *mm_broadband_modem_new (const gchar *device,
                                           const gchar **drivers,
@@ -93,17 +100,7 @@ MMBroadbandModem *mm_broadband_modem_new (const gchar *device,
                                           guint16 vendor_id,
                                           guint16 product_id);
 
-/* Convert the given string, which comes in the charset currently set in the
- * modem, to UTF-8. Given in the API so that subclasses can also use it directly.
- */
-gchar *mm_broadband_modem_take_and_convert_to_utf8 (MMBroadbandModem *self,
-                                                    gchar *str);
-
-/* Convert the given string, which comes in UTF-8, to the charset currently set
- * in the modem. Given in the API so that subclasses can also use it directly.
- */
-gchar *mm_broadband_modem_take_and_convert_to_current_charset (MMBroadbandModem *self,
-                                                               gchar *str);
+MMModemCharset mm_broadband_modem_get_current_charset (MMBroadbandModem *self);
 
 /* Create a unique device identifier string using the ATI and ATI1 replies and some
  * additional internal info */
@@ -123,5 +120,7 @@ gboolean mm_broadband_modem_lock_sms_storages_finish (MMBroadbandModem *self,
 void     mm_broadband_modem_unlock_sms_storages      (MMBroadbandModem *self,
                                                       gboolean mem1,
                                                       gboolean mem2);
+/* Helper to update SIM hot swap */
+void mm_broadband_modem_sim_hot_swap_detected (MMBroadbandModem *self);
 
 #endif /* MM_BROADBAND_MODEM_H */
