@@ -27,7 +27,7 @@
 #include <mm-errors-types.h>
 
 #include "mm-port-probe.h"
-#include "mm-log.h"
+#include "mm-log-object.h"
 #include "mm-port-serial-at.h"
 #include "mm-port-serial.h"
 #include "mm-serial-parsers.h"
@@ -63,7 +63,10 @@
  *   |----> MBIM capabilities check
  */
 
-G_DEFINE_TYPE (MMPortProbe, mm_port_probe, G_TYPE_OBJECT)
+static void log_object_iface_init (MMLogObjectInterface *iface);
+
+G_DEFINE_TYPE_EXTENDED (MMPortProbe, mm_port_probe, G_TYPE_OBJECT, 0,
+                        G_IMPLEMENT_INTERFACE (MM_TYPE_LOG_OBJECT, log_object_iface_init))
 
 enum {
     PROP_0,
@@ -93,10 +96,13 @@ struct _MMPortProbePrivate {
     /* From udev tags */
     gboolean is_ignored;
     gboolean is_gps;
+    gboolean is_audio;
     gboolean maybe_at_primary;
     gboolean maybe_at_secondary;
     gboolean maybe_at_ppp;
     gboolean maybe_qcdm;
+    gboolean maybe_qmi;
+    gboolean maybe_mbim;
 
     /* Current probing task. Only one can be available at a time */
     GTask *task;
@@ -158,9 +164,7 @@ mm_port_probe_set_result_at (MMPortProbe *self,
     self->priv->flags |= MM_PORT_PROBE_AT;
 
     if (self->priv->is_at) {
-        mm_dbg ("(%s/%s) port is AT-capable",
-                mm_kernel_device_get_subsystem (self->priv->port),
-                mm_kernel_device_get_name (self->priv->port));
+        mm_obj_dbg (self, "port is AT-capable");
 
         /* Also set as not a QCDM/QMI/MBIM port */
         self->priv->is_qcdm = FALSE;
@@ -168,9 +172,7 @@ mm_port_probe_set_result_at (MMPortProbe *self,
         self->priv->is_mbim = FALSE;
         self->priv->flags |= (MM_PORT_PROBE_QCDM | MM_PORT_PROBE_QMI | MM_PORT_PROBE_MBIM);
     } else {
-        mm_dbg ("(%s/%s) port is not AT-capable",
-                mm_kernel_device_get_subsystem (self->priv->port),
-                mm_kernel_device_get_name (self->priv->port));
+        mm_obj_dbg (self, "port is not AT-capable");
         self->priv->vendor = NULL;
         self->priv->product = NULL;
         self->priv->is_icera = FALSE;
@@ -187,15 +189,11 @@ mm_port_probe_set_result_at_vendor (MMPortProbe *self,
                                     const gchar *at_vendor)
 {
     if (at_vendor) {
-        mm_dbg ("(%s/%s) vendor probing finished",
-                mm_kernel_device_get_subsystem (self->priv->port),
-                mm_kernel_device_get_name (self->priv->port));
+        mm_obj_dbg (self, "vendor probing finished");
         self->priv->vendor = g_utf8_casefold (at_vendor, -1);
         self->priv->flags |= MM_PORT_PROBE_AT_VENDOR;
     } else {
-        mm_dbg ("(%s/%s) couldn't probe for vendor string",
-                mm_kernel_device_get_subsystem (self->priv->port),
-                mm_kernel_device_get_name (self->priv->port));
+        mm_obj_dbg (self, "couldn't probe for vendor string");
         self->priv->vendor = NULL;
         self->priv->product = NULL;
         self->priv->flags |= (MM_PORT_PROBE_AT_VENDOR | MM_PORT_PROBE_AT_PRODUCT);
@@ -207,15 +205,11 @@ mm_port_probe_set_result_at_product (MMPortProbe *self,
                                      const gchar *at_product)
 {
     if (at_product) {
-        mm_dbg ("(%s/%s) product probing finished",
-                mm_kernel_device_get_subsystem (self->priv->port),
-                mm_kernel_device_get_name (self->priv->port));
+        mm_obj_dbg (self, "product probing finished");
         self->priv->product = g_utf8_casefold (at_product, -1);
         self->priv->flags |= MM_PORT_PROBE_AT_PRODUCT;
     } else {
-        mm_dbg ("(%s/%s) couldn't probe for product string",
-                mm_kernel_device_get_subsystem (self->priv->port),
-                mm_kernel_device_get_name (self->priv->port));
+        mm_obj_dbg (self, "couldn't probe for product string");
         self->priv->product = NULL;
         self->priv->flags |= MM_PORT_PROBE_AT_PRODUCT;
     }
@@ -226,15 +220,11 @@ mm_port_probe_set_result_at_icera (MMPortProbe *self,
                                    gboolean is_icera)
 {
     if (is_icera) {
-        mm_dbg ("(%s/%s) Modem is Icera-based",
-                mm_kernel_device_get_subsystem (self->priv->port),
-                mm_kernel_device_get_name (self->priv->port));
+        mm_obj_dbg (self, "modem is Icera-based");
         self->priv->is_icera = TRUE;
         self->priv->flags |= MM_PORT_PROBE_AT_ICERA;
     } else {
-        mm_dbg ("(%s/%s) Modem is probably not Icera-based",
-                mm_kernel_device_get_subsystem (self->priv->port),
-                mm_kernel_device_get_name (self->priv->port));
+        mm_obj_dbg (self, "modem is probably not Icera-based");
         self->priv->is_icera = FALSE;
         self->priv->flags |= MM_PORT_PROBE_AT_ICERA;
     }
@@ -245,15 +235,11 @@ mm_port_probe_set_result_at_xmm (MMPortProbe *self,
                                  gboolean is_xmm)
 {
     if (is_xmm) {
-        mm_dbg ("(%s/%s) Modem is XMM-based",
-                mm_kernel_device_get_subsystem (self->priv->port),
-                mm_kernel_device_get_name (self->priv->port));
+        mm_obj_dbg (self, "modem is XMM-based");
         self->priv->is_xmm = TRUE;
         self->priv->flags |= MM_PORT_PROBE_AT_XMM;
     } else {
-        mm_dbg ("(%s/%s) Modem is probably not XMM-based",
-                mm_kernel_device_get_subsystem (self->priv->port),
-                mm_kernel_device_get_name (self->priv->port));
+        mm_obj_dbg (self, "modem is probably not XMM-based");
         self->priv->is_xmm = FALSE;
         self->priv->flags |= MM_PORT_PROBE_AT_XMM;
     }
@@ -267,9 +253,7 @@ mm_port_probe_set_result_qcdm (MMPortProbe *self,
     self->priv->flags |= MM_PORT_PROBE_QCDM;
 
     if (self->priv->is_qcdm) {
-        mm_dbg ("(%s/%s) port is QCDM-capable",
-                mm_kernel_device_get_subsystem (self->priv->port),
-                mm_kernel_device_get_name (self->priv->port));
+        mm_obj_dbg (self, "port is QCDM-capable");
 
         /* Also set as not an AT/QMI/MBIM port */
         self->priv->is_at = FALSE;
@@ -287,9 +271,7 @@ mm_port_probe_set_result_qcdm (MMPortProbe *self,
                               MM_PORT_PROBE_QMI |
                               MM_PORT_PROBE_MBIM);
     } else
-        mm_dbg ("(%s/%s) port is not QCDM-capable",
-                mm_kernel_device_get_subsystem (self->priv->port),
-                mm_kernel_device_get_name (self->priv->port));
+        mm_obj_dbg (self, "port is not QCDM-capable");
 }
 
 void
@@ -300,9 +282,7 @@ mm_port_probe_set_result_qmi (MMPortProbe *self,
     self->priv->flags |= MM_PORT_PROBE_QMI;
 
     if (self->priv->is_qmi) {
-        mm_dbg ("(%s/%s) port is QMI-capable",
-                mm_kernel_device_get_subsystem (self->priv->port),
-                mm_kernel_device_get_name (self->priv->port));
+        mm_obj_dbg (self, "port is QMI-capable");
 
         /* Also set as not an AT/QCDM/MBIM port */
         self->priv->is_at = FALSE;
@@ -318,9 +298,7 @@ mm_port_probe_set_result_qmi (MMPortProbe *self,
                               MM_PORT_PROBE_QCDM |
                               MM_PORT_PROBE_MBIM);
     } else
-        mm_dbg ("(%s/%s) port is not QMI-capable",
-                mm_kernel_device_get_subsystem (self->priv->port),
-                mm_kernel_device_get_name (self->priv->port));
+        mm_obj_dbg (self, "port is not QMI-capable");
 }
 
 void
@@ -331,9 +309,7 @@ mm_port_probe_set_result_mbim (MMPortProbe *self,
     self->priv->flags |= MM_PORT_PROBE_MBIM;
 
     if (self->priv->is_mbim) {
-        mm_dbg ("(%s/%s) port is MBIM-capable",
-                mm_kernel_device_get_subsystem (self->priv->port),
-                mm_kernel_device_get_name (self->priv->port));
+        mm_obj_dbg (self, "port is MBIM-capable");
 
         /* Also set as not an AT/QCDM/QMI port */
         self->priv->is_at = FALSE;
@@ -349,9 +325,7 @@ mm_port_probe_set_result_mbim (MMPortProbe *self,
                               MM_PORT_PROBE_QCDM |
                               MM_PORT_PROBE_QMI);
     } else
-        mm_dbg ("(%s/%s) port is not MBIM-capable",
-                mm_kernel_device_get_subsystem (self->priv->port),
-                mm_kernel_device_get_name (self->priv->port));
+        mm_obj_dbg (self, "port is not MBIM-capable");
 }
 
 /*****************************************************************************/
@@ -434,8 +408,8 @@ port_probe_run_context_free (PortProbeRunContext *ctx)
 
 #if defined WITH_QMI
     if (ctx->port_qmi) {
-        if (mm_port_qmi_is_open (ctx->port_qmi))
-            mm_port_qmi_close (ctx->port_qmi);
+        /* We should have closed it cleanly before */
+        g_assert (!mm_port_qmi_is_open (ctx->port_qmi));
         g_object_unref (ctx->port_qmi);
     }
 #endif
@@ -463,6 +437,22 @@ static gboolean wdm_probe (MMPortProbe *self);
 
 #if defined WITH_QMI
 
+static void
+qmi_port_close_ready (MMPortQmi    *qmi_port,
+                      GAsyncResult *res,
+                      MMPortProbe  *self)
+{
+    PortProbeRunContext *ctx;
+
+    g_assert (self->priv->task);
+    ctx = g_task_get_task_data (self->priv->task);
+
+    mm_port_qmi_close_finish (qmi_port, res, NULL);
+
+    /* Keep on */
+    ctx->source_id = g_idle_add ((GSourceFunc) wdm_probe, self);
+}
+
 static void
 port_qmi_open_ready (MMPortQmi    *port_qmi,
                      GAsyncResult *res,
@@ -477,19 +467,17 @@ port_qmi_open_ready (MMPortQmi    *port_qmi,
 
     is_qmi = mm_port_qmi_open_finish (port_qmi, res, &error);
     if (!is_qmi) {
-        mm_dbg ("(%s/%s) error checking QMI support: '%s'",
-                mm_kernel_device_get_subsystem (self->priv->port),
-                mm_kernel_device_get_name (self->priv->port),
-                error ? error->message : "unknown error");
+        mm_obj_dbg (self, "error checking QMI support: %s",
+                    error ? error->message : "unknown error");
         g_clear_error (&error);
     }
 
     /* Set probing result */
     mm_port_probe_set_result_qmi (self, is_qmi);
-    mm_port_qmi_close (port_qmi);
 
-    /* Keep on */
-    ctx->source_id = g_idle_add ((GSourceFunc) wdm_probe, self);
+    mm_port_qmi_close (ctx->port_qmi,
+                       (GAsyncReadyCallback) qmi_port_close_ready,
+                       self);
 }
 
 #endif /* WITH_QMI */
@@ -503,17 +491,22 @@ wdm_probe_qmi (MMPortProbe *self)
     ctx = g_task_get_task_data (self->priv->task);
 
 #if defined WITH_QMI
-    mm_dbg ("(%s/%s) probing QMI...",
-            mm_kernel_device_get_subsystem (self->priv->port),
-            mm_kernel_device_get_name (self->priv->port));
+    {
+        MMPortSubsys subsys = MM_PORT_SUBSYS_USBMISC;
 
-    /* Create a port and try to open it */
-    ctx->port_qmi = mm_port_qmi_new (mm_kernel_device_get_name (self->priv->port));
-    mm_port_qmi_open (ctx->port_qmi,
-                      FALSE,
-                      NULL,
-                      (GAsyncReadyCallback) port_qmi_open_ready,
-                      self);
+        mm_obj_dbg (self, "probing QMI...");
+
+        if (g_str_equal (mm_kernel_device_get_subsystem (self->priv->port), "rpmsg"))
+            subsys = MM_PORT_SUBSYS_RPMSG;
+
+        /* Create a port and try to open it */
+        ctx->port_qmi = mm_port_qmi_new (mm_kernel_device_get_name (self->priv->port), subsys);
+        mm_port_qmi_open (ctx->port_qmi,
+                          FALSE,
+                          NULL,
+                          (GAsyncReadyCallback) port_qmi_open_ready,
+                          self);
+    }
 #else
     /* If not compiled with QMI support, just assume we won't have any QMI port */
     mm_port_probe_set_result_qmi (self, FALSE);
@@ -553,10 +546,8 @@ mbim_port_open_ready (MMPortMbim   *mbim_port,
 
     is_mbim = mm_port_mbim_open_finish (mbim_port, res, &error);
     if (!is_mbim) {
-        mm_dbg ("(%s/%s) error checking MBIM support: '%s'",
-                mm_kernel_device_get_subsystem (self->priv->port),
-                mm_kernel_device_get_name (self->priv->port),
-                error ? error->message : "unknown error");
+        mm_obj_dbg (self, "error checking MBIM support: %s",
+                    error ? error->message : "unknown error");
         g_clear_error (&error);
     }
 
@@ -579,12 +570,11 @@ wdm_probe_mbim (MMPortProbe *self)
     ctx = g_task_get_task_data (self->priv->task);
 
 #if defined WITH_MBIM
-    mm_dbg ("(%s/%s) probing MBIM...",
-            mm_kernel_device_get_subsystem (self->priv->port),
-            mm_kernel_device_get_name (self->priv->port));
+    mm_obj_dbg (self, "probing MBIM...");
 
     /* Create a port and try to open it */
-    ctx->mbim_port = mm_port_mbim_new (mm_kernel_device_get_name (self->priv->port));
+    ctx->mbim_port = mm_port_mbim_new (mm_kernel_device_get_name (self->priv->port),
+                                       MM_PORT_SUBSYS_USBMISC);
     mm_port_mbim_open (ctx->mbim_port,
 #if defined WITH_QMI && QMI_MBIM_QMUX_SUPPORTED
                        FALSE, /* Don't check QMI over MBIM support at this stage */
@@ -651,10 +641,7 @@ common_serial_port_setup (MMPortProbe  *self,
 
         flow_control = mm_flow_control_from_string (flow_control_tag, &error);
         if (flow_control == MM_FLOW_CONTROL_UNKNOWN) {
-            mm_warn ("(%s/%s) Unsupported flow control settings in port: %s",
-                     mm_kernel_device_get_subsystem (self->priv->port),
-                     mm_kernel_device_get_name (self->priv->port),
-                     error->message);
+            mm_obj_warn (self, "unsupported flow control settings in port: %s", error->message);
             g_error_free (error);
         } else {
             g_object_set (serial,
@@ -691,10 +678,7 @@ serial_probe_qcdm_parse_response (MMPortSerialQcdm *port,
         /* Parse the response */
         result = qcdm_cmd_version_info_result ((const gchar *) response->data, response->len, &err);
         if (!result) {
-            mm_warn ("(%s/%s) failed to parse QCDM version info command result: %d",
-                     mm_kernel_device_get_subsystem (self->priv->port),
-                     mm_kernel_device_get_name (self->priv->port),
-                     err);
+            mm_obj_warn (self, "failed to parse QCDM version info command result: %d", err);
             retry = TRUE;
         } else {
             /* yay, probably a QCDM port */
@@ -704,11 +688,11 @@ serial_probe_qcdm_parse_response (MMPortSerialQcdm *port,
         g_byte_array_unref (response);
     } else if (g_error_matches (error, MM_SERIAL_ERROR, MM_SERIAL_ERROR_PARSE_FAILED)) {
         /* Failed to unescape QCDM packet: don't retry */
-        mm_dbg ("QCDM parsing error: %s", error->message);
+        mm_obj_dbg (self, "QCDM parsing error: %s", error->message);
         g_error_free (error);
     } else {
         if (!g_error_matches (error, MM_SERIAL_ERROR, MM_SERIAL_ERROR_RESPONSE_TIMEOUT))
-            mm_dbg ("QCDM probe error: (%d) %s", error->code, error->message);
+            mm_obj_dbg (self, "QCDM probe error: (%d) %s", error->code, error->message);
         g_error_free (error);
         retry = TRUE;
     }
@@ -755,9 +739,7 @@ serial_probe_qcdm (MMPortProbe *self)
     if (port_probe_task_return_error_if_cancelled (self))
         return G_SOURCE_REMOVE;
 
-    mm_dbg ("(%s/%s) probing QCDM...",
-            mm_kernel_device_get_subsystem (self->priv->port),
-            mm_kernel_device_get_name (self->priv->port));
+    mm_obj_dbg (self, "probing QCDM...");
 
     /* If open, close the AT port */
     if (ctx->serial) {
@@ -771,7 +753,7 @@ serial_probe_qcdm (MMPortProbe *self)
     }
 
     /* Open the QCDM port */
-    ctx->serial = MM_PORT_SERIAL (mm_port_serial_qcdm_new (mm_kernel_device_get_name (self->priv->port)));
+    ctx->serial = MM_PORT_SERIAL (mm_port_serial_qcdm_new (mm_kernel_device_get_name (self->priv->port), MM_PORT_SUBSYS_TTY));
     if (!ctx->serial) {
         port_probe_task_return_error (self,
                                       g_error_new (MM_CORE_ERROR,
@@ -986,9 +968,7 @@ serial_probe_at_parse_response (MMPortSerialAt *port,
 
     /* If AT probing cancelled, end this partial probing */
     if (g_cancellable_is_cancelled (ctx->at_probing_cancellable)) {
-        mm_dbg ("(%s/%s) no need to keep on probing the port for AT support",
-                mm_kernel_device_get_subsystem (self->priv->port),
-                mm_kernel_device_get_name (self->priv->port));
+        mm_obj_dbg (self, "no need to keep on probing the port for AT support");
         ctx->at_result_processor (self, NULL);
         serial_probe_schedule (self);
         return;
@@ -1029,10 +1009,8 @@ serial_probe_at_parse_response (MMPortSerialAt *port,
         if (ctx->at_commands_wait_secs == 0)
             ctx->source_id = g_idle_add ((GSourceFunc) serial_probe_at, self);
         else {
-            mm_dbg ("(%s/%s) re-scheduling next command in probing group in %u seconds...",
-                    mm_kernel_device_get_subsystem (self->priv->port),
-                    mm_kernel_device_get_name (self->priv->port),
-                    ctx->at_commands_wait_secs);
+            mm_obj_dbg (self, "re-scheduling next command in probing group in %u seconds...",
+                        ctx->at_commands_wait_secs);
             ctx->source_id = g_timeout_add_seconds (ctx->at_commands_wait_secs, (GSourceFunc) serial_probe_at, self);
         }
         goto out;
@@ -1066,9 +1044,7 @@ serial_probe_at (MMPortProbe *self)
 
     /* If AT probing cancelled, end this partial probing */
     if (g_cancellable_is_cancelled (ctx->at_probing_cancellable)) {
-        mm_dbg ("(%s/%s) no need to launch probing for AT support",
-                mm_kernel_device_get_subsystem (self->priv->port),
-                mm_kernel_device_get_name (self->priv->port));
+        mm_obj_dbg (self, "no need to launch probing for AT support");
         ctx->at_result_processor (self, NULL);
         serial_probe_schedule (self);
         return G_SOURCE_REMOVE;
@@ -1258,9 +1234,7 @@ serial_buffer_full (MMPortSerial *serial,
     g_assert (self->priv->task);
     ctx = g_task_get_task_data (self->priv->task);
 
-    mm_dbg ("(%s/%s) serial buffer full",
-            mm_kernel_device_get_subsystem (self->priv->port),
-            mm_kernel_device_get_name (self->priv->port));
+    mm_obj_dbg (self, "serial buffer full");
     /* Don't explicitly close the AT port, just end the AT probing
      * (or custom init probing) */
     mm_port_probe_set_result_at (self, FALSE);
@@ -1303,8 +1277,10 @@ serial_open_at (MMPortProbe *self)
         gpointer parser;
         MMPortSubsys subsys = MM_PORT_SUBSYS_TTY;
 
-        if (g_str_has_prefix (mm_kernel_device_get_subsystem (self->priv->port), "usb"))
-            subsys = MM_PORT_SUBSYS_USB;
+        if (g_str_equal (mm_kernel_device_get_subsystem (self->priv->port), "usbmisc"))
+            subsys = MM_PORT_SUBSYS_USBMISC;
+        else if (g_str_equal (mm_kernel_device_get_subsystem (self->priv->port), "rpmsg"))
+            subsys = MM_PORT_SUBSYS_RPMSG;
 
         ctx->serial = MM_PORT_SERIAL (mm_port_serial_at_new (mm_kernel_device_get_name (self->priv->port), subsys));
         if (!ctx->serial) {
@@ -1403,9 +1379,7 @@ mm_port_probe_run_cancel_at_probing (MMPortProbe *self)
     if (g_cancellable_is_cancelled (ctx->at_probing_cancellable))
         return FALSE;
 
-    mm_dbg ("(%s/%s) requested to cancel all AT probing",
-            mm_kernel_device_get_subsystem (self->priv->port),
-            mm_kernel_device_get_name (self->priv->port));
+    mm_obj_dbg (self, "requested to cancel all AT probing");
     g_cancellable_cancel (ctx->at_probing_cancellable);
     return TRUE;
 }
@@ -1461,36 +1435,59 @@ mm_port_probe_run (MMPortProbe                *self,
 
     /* If we're told to completely ignore the port, don't do any probing */
     if (self->priv->is_ignored) {
-        mm_dbg ("(%s/%s) port probing finished: skipping for blacklisted port",
-                mm_kernel_device_get_subsystem (self->priv->port),
-                mm_kernel_device_get_name (self->priv->port));
+        mm_obj_dbg (self, "port probing finished: skipping for blacklisted port");
         port_probe_task_return_boolean (self, TRUE);
         return;
     }
 
-    /* If this is a port flagged as a GPS port, don't do any AT or QCDM probing */
+    /* If this is a port flagged as a GPS port, don't do any other probing */
     if (self->priv->is_gps) {
-        mm_dbg ("(%s/%s) GPS port detected",
-                mm_kernel_device_get_subsystem (self->priv->port),
-                mm_kernel_device_get_name (self->priv->port));
-        mm_port_probe_set_result_at (self, FALSE);
+        mm_obj_dbg (self, "GPS port detected");
+        mm_port_probe_set_result_at   (self, FALSE);
         mm_port_probe_set_result_qcdm (self, FALSE);
+        mm_port_probe_set_result_qmi  (self, FALSE);
+        mm_port_probe_set_result_mbim (self, FALSE);
     }
 
-    /* If this is a port flagged as being an AT port, don't do any QCDM probing */
+    /* If this is a port flagged as an audio port, don't do any other probing */
+    if (self->priv->is_audio) {
+        mm_obj_dbg (self, "audio port detected");
+        mm_port_probe_set_result_at   (self, FALSE);
+        mm_port_probe_set_result_qcdm (self, FALSE);
+        mm_port_probe_set_result_qmi  (self, FALSE);
+        mm_port_probe_set_result_mbim (self, FALSE);
+    }
+
+    /* If this is a port flagged as being an AT port, don't do any other probing */
     if (self->priv->maybe_at_primary || self->priv->maybe_at_secondary || self->priv->maybe_at_ppp) {
-        mm_dbg ("(%s/%s) no QCDM probing in possible AT port",
-                mm_kernel_device_get_subsystem (self->priv->port),
-                mm_kernel_device_get_name (self->priv->port));
+        mm_obj_dbg (self, "no QCDM/QMI/MBIM probing in possible AT port");
         mm_port_probe_set_result_qcdm (self, FALSE);
+        mm_port_probe_set_result_qmi  (self, FALSE);
+        mm_port_probe_set_result_mbim (self, FALSE);
     }
 
-    /* If this is a port flagged as being a QCDM port, don't do any AT probing */
+    /* If this is a port flagged as being a QCDM port, don't do any other probing */
     if (self->priv->maybe_qcdm) {
-        mm_dbg ("(%s/%s) no AT probing in possible QCDM port",
-                mm_kernel_device_get_subsystem (self->priv->port),
-                mm_kernel_device_get_name (self->priv->port));
-        mm_port_probe_set_result_at (self, FALSE);
+        mm_obj_dbg (self, "no AT/QMI/MBIM probing in possible QCDM port");
+        mm_port_probe_set_result_at   (self, FALSE);
+        mm_port_probe_set_result_qmi  (self, FALSE);
+        mm_port_probe_set_result_mbim (self, FALSE);
+    }
+
+    /* If this is a port flagged as being a QMI port, don't do any other probing */
+    if (self->priv->maybe_qmi) {
+        mm_obj_dbg (self, "no AT/QCDM/MBIM probing in possible QMI port");
+        mm_port_probe_set_result_at   (self, FALSE);
+        mm_port_probe_set_result_qcdm (self, FALSE);
+        mm_port_probe_set_result_mbim (self, FALSE);
+    }
+
+    /* If this is a port flagged as being a MBIM port, don't do any other probing */
+    if (self->priv->maybe_mbim) {
+        mm_obj_dbg (self, "no AT/QCDM/QMI probing in possible MBIM port");
+        mm_port_probe_set_result_at   (self, FALSE);
+        mm_port_probe_set_result_qcdm (self, FALSE);
+        mm_port_probe_set_result_qmi  (self, FALSE);
     }
 
     /* Check if we already have the requested probing results.
@@ -1503,19 +1500,14 @@ mm_port_probe_run (MMPortProbe                *self,
 
     /* All requested probings already available? If so, we're done */
     if (!ctx->flags) {
-        mm_dbg ("(%s/%s) port probing finished: no more probings needed",
-                mm_kernel_device_get_subsystem (self->priv->port),
-                mm_kernel_device_get_name (self->priv->port));
+        mm_obj_dbg (self, "port probing finished: no more probings needed");
         port_probe_task_return_boolean (self, TRUE);
         return;
     }
 
     /* Log the probes scheduled to be run */
     probe_list_str = mm_port_probe_flag_build_string_from_mask (ctx->flags);
-    mm_dbg ("(%s/%s) launching port probing: '%s'",
-            mm_kernel_device_get_subsystem (self->priv->port),
-            mm_kernel_device_get_name (self->priv->port),
-            probe_list_str);
+    mm_obj_dbg (self, "launching port probing: '%s'", probe_list_str);
     g_free (probe_list_str);
 
     /* If any AT probing is needed, start by opening as AT port */
@@ -1587,18 +1579,8 @@ mm_port_probe_list_has_at_port (GList *list)
 gboolean
 mm_port_probe_is_qcdm (MMPortProbe *self)
 {
-    const gchar *subsys;
-    const gchar *name;
-
     g_return_val_if_fail (MM_IS_PORT_PROBE (self), FALSE);
 
-    subsys = mm_kernel_device_get_subsystem (self->priv->port);
-    name = mm_kernel_device_get_name (self->priv->port);
-    if (g_str_equal (subsys, "net") ||
-        (g_str_has_prefix (subsys, "usb") &&
-         g_str_has_prefix (name, "cdc-wdm")))
-        return FALSE;
-
     return (self->priv->flags & MM_PORT_PROBE_QCDM ?
             self->priv->is_qcdm :
             FALSE);
@@ -1607,19 +1589,11 @@ mm_port_probe_is_qcdm (MMPortProbe *self)
 gboolean
 mm_port_probe_is_qmi (MMPortProbe *self)
 {
-    const gchar *subsys;
-    const gchar *name;
-
     g_return_val_if_fail (MM_IS_PORT_PROBE (self), FALSE);
 
-    subsys = mm_kernel_device_get_subsystem (self->priv->port);
-    name = mm_kernel_device_get_name (self->priv->port);
-    if (!g_str_has_prefix (subsys, "usb") ||
-        !name ||
-        !g_str_has_prefix (name, "cdc-wdm"))
-        return FALSE;
-
-    return self->priv->is_qmi;
+    return (self->priv->flags & MM_PORT_PROBE_QMI ?
+            self->priv->is_qmi :
+            FALSE);
 }
 
 gboolean
@@ -1641,19 +1615,11 @@ mm_port_probe_list_has_qmi_port (GList *list)
 gboolean
 mm_port_probe_is_mbim (MMPortProbe *self)
 {
-    const gchar *subsys;
-    const gchar *name;
-
     g_return_val_if_fail (MM_IS_PORT_PROBE (self), FALSE);
 
-    subsys = mm_kernel_device_get_subsystem (self->priv->port);
-    name = mm_kernel_device_get_name (self->priv->port);
-    if (!g_str_has_prefix (subsys, "usb") ||
-        !name ||
-        !g_str_has_prefix (name, "cdc-wdm"))
-        return FALSE;
-
-    return self->priv->is_mbim;
+    return (self->priv->flags & MM_PORT_PROBE_MBIM ?
+            self->priv->is_mbim :
+            FALSE);
 }
 
 gboolean
@@ -1676,26 +1642,22 @@ MMPortType
 mm_port_probe_get_port_type (MMPortProbe *self)
 {
     const gchar *subsys;
-    const gchar *name;
 
     g_return_val_if_fail (MM_IS_PORT_PROBE (self), FALSE);
 
     subsys = mm_kernel_device_get_subsystem (self->priv->port);
-    name = mm_kernel_device_get_name (self->priv->port);
 
     if (g_str_equal (subsys, "net"))
         return MM_PORT_TYPE_NET;
 
 #if defined WITH_QMI
-    if (g_str_has_prefix (subsys, "usb") &&
-        g_str_has_prefix (name, "cdc-wdm") &&
+    if (self->priv->flags & MM_PORT_PROBE_QMI &&
         self->priv->is_qmi)
         return MM_PORT_TYPE_QMI;
 #endif
 
 #if defined WITH_MBIM
-    if (g_str_has_prefix (subsys, "usb") &&
-        g_str_has_prefix (name, "cdc-wdm") &&
+    if (self->priv->flags & MM_PORT_PROBE_MBIM &&
         self->priv->is_mbim)
         return MM_PORT_TYPE_MBIM;
 #endif
@@ -1711,6 +1673,9 @@ mm_port_probe_get_port_type (MMPortProbe *self)
     if (self->priv->is_gps)
         return MM_PORT_TYPE_GPS;
 
+    if (self->priv->is_audio)
+        return MM_PORT_TYPE_AUDIO;
+
     return MM_PORT_TYPE_UNKNOWN;
 }
 
@@ -1749,18 +1714,8 @@ mm_port_probe_get_port (MMPortProbe *self)
 const gchar *
 mm_port_probe_get_vendor (MMPortProbe *self)
 {
-    const gchar *subsys;
-    const gchar *name;
-
     g_return_val_if_fail (MM_IS_PORT_PROBE (self), FALSE);
 
-    subsys = mm_kernel_device_get_subsystem (self->priv->port);
-    name = mm_kernel_device_get_name (self->priv->port);
-    if (g_str_equal (subsys, "net") ||
-        (g_str_has_prefix (subsys, "usb") &&
-         g_str_has_prefix (name, "cdc-wdm")))
-        return NULL;
-
     return (self->priv->flags & MM_PORT_PROBE_AT_VENDOR ?
             self->priv->vendor :
             NULL);
@@ -1769,18 +1724,8 @@ mm_port_probe_get_vendor (MMPortProbe *self)
 const gchar *
 mm_port_probe_get_product (MMPortProbe *self)
 {
-    const gchar *subsys;
-    const gchar *name;
-
     g_return_val_if_fail (MM_IS_PORT_PROBE (self), FALSE);
 
-    subsys = mm_kernel_device_get_subsystem (self->priv->port);
-    name = mm_kernel_device_get_name (self->priv->port);
-    if (g_str_equal (subsys, "net") ||
-        (g_str_has_prefix (subsys, "usb") &&
-         g_str_has_prefix (name, "cdc-wdm")))
-        return NULL;
-
     return (self->priv->flags & MM_PORT_PROBE_AT_PRODUCT ?
             self->priv->product :
             NULL);
@@ -1791,9 +1736,6 @@ mm_port_probe_is_icera (MMPortProbe *self)
 {
     g_return_val_if_fail (MM_IS_PORT_PROBE (self), FALSE);
 
-    if (g_str_equal (mm_kernel_device_get_subsystem (self->priv->port), "net"))
-        return FALSE;
-
     return (self->priv->flags & MM_PORT_PROBE_AT_ICERA ?
             self->priv->is_icera :
             FALSE);
@@ -1817,9 +1759,6 @@ mm_port_probe_is_xmm (MMPortProbe *self)
 {
     g_return_val_if_fail (MM_IS_PORT_PROBE (self), FALSE);
 
-    if (g_str_equal (mm_kernel_device_get_subsystem (self->priv->port), "net"))
-        return FALSE;
-
     return (self->priv->flags & MM_PORT_PROBE_AT_XMM ?
             self->priv->is_xmm :
             FALSE);
@@ -1864,6 +1803,17 @@ mm_port_probe_get_port_subsys (MMPortProbe *self)
 
 /*****************************************************************************/
 
+static gchar *
+log_object_build_id (MMLogObject *_self)
+{
+    MMPortProbe *self;
+
+    self = MM_PORT_PROBE (_self);
+    return g_strdup_printf ("%s/probe", mm_kernel_device_get_name (self->priv->port));
+}
+
+/*****************************************************************************/
+
 MMPortProbe *
 mm_port_probe_new (MMDevice       *device,
                    MMKernelDevice *port)
@@ -1900,10 +1850,13 @@ set_property (GObject *object,
         self->priv->port = g_value_dup_object (value);
         self->priv->is_ignored = mm_kernel_device_get_property_as_boolean (self->priv->port, ID_MM_PORT_IGNORE);
         self->priv->is_gps = mm_kernel_device_get_property_as_boolean (self->priv->port, ID_MM_PORT_TYPE_GPS);
+        self->priv->is_audio = mm_kernel_device_get_property_as_boolean (self->priv->port, ID_MM_PORT_TYPE_AUDIO);
         self->priv->maybe_at_primary = mm_kernel_device_get_property_as_boolean (self->priv->port, ID_MM_PORT_TYPE_AT_PRIMARY);
         self->priv->maybe_at_secondary = mm_kernel_device_get_property_as_boolean (self->priv->port, ID_MM_PORT_TYPE_AT_SECONDARY);
         self->priv->maybe_at_ppp = mm_kernel_device_get_property_as_boolean (self->priv->port, ID_MM_PORT_TYPE_AT_PPP);
         self->priv->maybe_qcdm = mm_kernel_device_get_property_as_boolean (self->priv->port, ID_MM_PORT_TYPE_QCDM);
+        self->priv->maybe_qmi = mm_kernel_device_get_property_as_boolean (self->priv->port, ID_MM_PORT_TYPE_QMI);
+        self->priv->maybe_mbim = mm_kernel_device_get_property_as_boolean (self->priv->port, ID_MM_PORT_TYPE_MBIM);
         break;
     default:
         G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
@@ -1959,6 +1912,12 @@ dispose (GObject *object)
     G_OBJECT_CLASS (mm_port_probe_parent_class)->dispose (object);
 }
 
+static void
+log_object_iface_init (MMLogObjectInterface *iface)
+{
+    iface->build_id = log_object_build_id;
+}
+
 static void
 mm_port_probe_class_init (MMPortProbeClass *klass)
 {
