@@ -120,23 +120,26 @@ sms_string_to_bcd_semi_octets (guint8 *buf, gsize buflen, const char *string)
 }
 
 /* len is in semi-octets */
-static char *
-sms_decode_address (const guint8 *address, int len)
+static gchar *
+sms_decode_address (const guint8  *address,
+                    gint           len,
+                    GError       **error)
 {
     guint8 addrtype, addrplan;
-    char *utf8;
+    gchar *utf8;
 
     addrtype = address[0] & SMS_NUMBER_TYPE_MASK;
     addrplan = address[0] & SMS_NUMBER_PLAN_MASK;
     address++;
 
     if (addrtype == SMS_NUMBER_TYPE_ALPHA) {
-        guint8 *unpacked;
-        guint32 unpacked_len;
+        g_autoptr(GByteArray)  unpacked_array = NULL;
+        guint8                *unpacked = NULL;
+        guint32                unpacked_len;
+
         unpacked = mm_charset_gsm_unpack (address, (len * 4) / 7, 0, &unpacked_len);
-        utf8 = (char *)mm_charset_gsm_unpacked_to_utf8 (unpacked,
-                                                        unpacked_len);
-        g_free (unpacked);
+        unpacked_array = g_byte_array_new_take (unpacked, unpacked_len);
+        utf8 = mm_modem_charset_bytearray_to_utf8 (unpacked_array, MM_MODEM_CHARSET_GSM, FALSE, error);
     } else if (addrtype == SMS_NUMBER_TYPE_INTL &&
                addrplan == SMS_NUMBER_PLAN_TELEPHONE) {
         /* International telphone number, format as "+1234567890" */
@@ -156,25 +159,26 @@ sms_decode_address (const guint8 *address, int len)
     return utf8;
 }
 
-static char *
+static gchar *
 sms_decode_timestamp (const guint8 *timestamp)
 {
-    /* YYMMDDHHMMSS+ZZ */
-    char *timestr;
-    int quarters, hours;
+    /* ISO8601 format: YYYY-MM-DDTHH:MM:SS+HHMM */
+    guint year, month, day, hour, minute, second;
+    gint quarters, offset_minutes;
 
-    timestr = g_malloc0 (16);
-    sms_semi_octets_to_bcd_string (timestr, timestamp, 6);
+    year = 2000 + ((timestamp[0] & 0xf) * 10) + ((timestamp[0] >> 4) & 0xf);
+    month = ((timestamp[1] & 0xf) * 10) + ((timestamp[1] >> 4) & 0xf);
+    day = ((timestamp[2] & 0xf) * 10) + ((timestamp[2] >> 4) & 0xf);
+    hour = ((timestamp[3] & 0xf) * 10) + ((timestamp[3] >> 4) & 0xf);
+    minute = ((timestamp[4] & 0xf) * 10) + ((timestamp[4] >> 4) & 0xf);
+    second = ((timestamp[5] & 0xf) * 10) + ((timestamp[5] >> 4) & 0xf);
     quarters = ((timestamp[6] & 0x7) * 10) + ((timestamp[6] >> 4) & 0xf);
-    hours = quarters / 4;
+    offset_minutes = quarters * 15;
     if (timestamp[6] & 0x08)
-        timestr[12] = '-';
-    else
-        timestr[12] = '+';
-    timestr[13] = (hours / 10) + '0';
-    timestr[14] = (hours % 10) + '0';
-    /* TODO(njw): Change timestamp rep to something that includes quarter-hours */
-    return timestr;
+        offset_minutes = -1 * offset_minutes;
+
+    return mm_new_iso8601_time (year, month, day, hour,
+                                minute, second, TRUE, offset_minutes);
 }
 
 static MMSmsEncoding
@@ -191,14 +195,16 @@ sms_encoding_type (int dcs)
                     scheme = MM_SMS_ENCODING_UCS2;
                     break;
                 case 0x00:
-                    /* fallthrough */
                     /* reserved - spec says to treat it as default alphabet */
+                    /* Fall through */
                 case 0x0c:
                     scheme = MM_SMS_ENCODING_GSM7;
                     break;
                 case 0x04:
                     scheme = MM_SMS_ENCODING_8BIT;
                     break;
+                default:
+                    g_assert_not_reached ();
             }
             break;
 
@@ -222,6 +228,8 @@ sms_encoding_type (int dcs)
                 case 0x04:
                     scheme = MM_SMS_ENCODING_8BIT;
                     break;
+                default:
+                    g_assert_not_reached ();
             }
             break;
 
@@ -234,50 +242,44 @@ sms_encoding_type (int dcs)
     return scheme;
 }
 
-static char *
-sms_decode_text (const guint8 *text, int len, MMSmsEncoding encoding, int bit_offset)
+static gchar *
+sms_decode_text (const guint8   *text,
+                 int             len,
+                 MMSmsEncoding   encoding,
+                 int             bit_offset,
+                 gpointer        log_object,
+                 GError        **error)
 {
-    char *utf8;
-    guint8 *unpacked;
-    guint32 unpacked_len;
-
     if (encoding == MM_SMS_ENCODING_GSM7) {
-        mm_dbg ("Converting SMS part text from GSM-7 to UTF-8...");
+        g_autoptr(GByteArray)  unpacked_array = NULL;
+        guint8                *unpacked = NULL;
+        guint32                unpacked_len;
+        gchar                 *utf8;
+
         unpacked = mm_charset_gsm_unpack ((const guint8 *) text, len, bit_offset, &unpacked_len);
-        utf8 = (char *) mm_charset_gsm_unpacked_to_utf8 (unpacked, unpacked_len);
-        mm_dbg ("   Got UTF-8 text: '%s'", utf8);
-        g_free (unpacked);
-    } else if (encoding == MM_SMS_ENCODING_UCS2) {
-        /* Despite 3GPP TS 23.038 specifies that Unicode SMS messages are
-         * encoded in UCS-2, UTF-16 encoding is commonly used instead on many
-         * modern platforms to allow encoding code points that fall outside the
-         * Basic Multilingual Plane (BMP), such as Emoji. Most of the UCS-2
-         * code points are identical to their equivalent UTF-16 code points.
-         * In UTF-16, non-BMP code points are encoded in a pair of surrogate
-         * code points (i.e. a high surrogate in 0xD800..0xDBFF, followed by a
-         * low surrogate in 0xDC00..0xDFFF). An isolated surrogate code point
-         * has no general interpretation in UTF-16, but could be a valid
-         * (though unmapped) code point in UCS-2. Here we first try to decode
-         * the SMS message in UTF-16BE, and if that fails, fall back to decode
-         * in UCS-2BE.
-         */
-        mm_dbg ("Converting SMS part text from UTF-16BE to UTF-8...");
-        utf8 = g_convert ((const gchar *) text, len, "UTF8", "UTF16BE", NULL, NULL, NULL);
-        if (!utf8) {
-            mm_dbg ("Converting SMS part text from UCS-2BE to UTF8...");
-            utf8 = g_convert ((const gchar *) text, len, "UTF8", "UCS-2BE", NULL, NULL, NULL);
-        }
-        if (!utf8) {
-            mm_warn ("Couldn't convert SMS part contents from UTF-16BE/UCS-2BE to UTF-8: not decoding any text");
-            utf8 = g_strdup ("");
-        } else
-            mm_dbg ("   Got UTF-8 text: '%s'", utf8);
-    } else {
-        mm_warn ("Unexpected encoding '%s': not decoding any text", mm_sms_encoding_get_string (encoding));
-        utf8 = g_strdup ("");
+        unpacked_array = g_byte_array_new_take (unpacked, unpacked_len);
+        utf8 = mm_modem_charset_bytearray_to_utf8 (unpacked_array, MM_MODEM_CHARSET_GSM, FALSE, error);
+        if (utf8)
+            mm_obj_dbg (log_object, "converted SMS part text from GSM-7 to UTF-8: %s", utf8);
+        return utf8;
     }
 
-    return utf8;
+    /* Always assume UTF-16 instead of UCS-2! */
+    if (encoding == MM_SMS_ENCODING_UCS2) {
+        g_autoptr(GByteArray)  bytearray = NULL;
+        gchar                 *utf8;
+
+        bytearray = g_byte_array_append (g_byte_array_sized_new (len), (const guint8 *)text, len);
+        utf8 = mm_modem_charset_bytearray_to_utf8 (bytearray, MM_MODEM_CHARSET_UTF16, FALSE, error);
+        if (utf8)
+            mm_obj_dbg (log_object, "converted SMS part text from UTF-16BE to UTF-8: %s", utf8);
+        return utf8;
+    }
+
+    g_set_error (error, MM_CORE_ERROR, MM_CORE_ERROR_FAILED,
+                 "Couldn't convert SMS part contents from %s to UTF-8",
+                 mm_sms_encoding_get_string (encoding));
+    return NULL;
 }
 
 static guint
@@ -338,35 +340,30 @@ validity_to_relative (guint validity)
 }
 
 MMSmsPart *
-mm_sms_part_3gpp_new_from_pdu (guint index,
-                               const gchar *hexpdu,
-                               GError **error)
+mm_sms_part_3gpp_new_from_pdu (guint         index,
+                               const gchar  *hexpdu,
+                               gpointer      log_object,
+                               GError      **error)
 {
-    gsize pdu_len;
-    guint8 *pdu;
-    MMSmsPart *part;
+    g_autofree guint8 *pdu = NULL;
+    gsize              pdu_len;
 
     /* Convert PDU from hex to binary */
-    pdu = (guint8 *) mm_utils_hexstr2bin (hexpdu, &pdu_len);
+    pdu = mm_utils_hexstr2bin (hexpdu, -1, &pdu_len, error);
     if (!pdu) {
-        g_set_error_literal (error,
-                             MM_CORE_ERROR,
-                             MM_CORE_ERROR_FAILED,
-                             "Couldn't convert 3GPP PDU from hex to binary");
+        g_prefix_error (error, "Couldn't convert 3GPP PDU from hex to binary: ");
         return NULL;
     }
 
-    part = mm_sms_part_3gpp_new_from_binary_pdu (index, pdu, pdu_len, error);
-    g_free (pdu);
-
-    return part;
+    return mm_sms_part_3gpp_new_from_binary_pdu (index, pdu, pdu_len, log_object, error);
 }
 
 MMSmsPart *
-mm_sms_part_3gpp_new_from_binary_pdu (guint index,
-                                      const guint8 *pdu,
-                                      gsize pdu_len,
-                                      GError **error)
+mm_sms_part_3gpp_new_from_binary_pdu (guint         index,
+                                      const guint8  *pdu,
+                                      gsize          pdu_len,
+                                      gpointer       log_object,
+                                      GError       **error)
 {
     MMSmsPart *sms_part;
     guint8 pdu_type;
@@ -382,14 +379,15 @@ mm_sms_part_3gpp_new_from_binary_pdu (guint index,
     guint tp_dcs_offset = 0;
     guint tp_user_data_len_offset = 0;
     MMSmsEncoding user_data_encoding = MM_SMS_ENCODING_UNKNOWN;
+    gchar *address;
 
     /* Create the new MMSmsPart */
     sms_part = mm_sms_part_new (index, MM_SMS_PDU_TYPE_UNKNOWN);
 
     if (index != SMS_PART_INVALID_INDEX)
-        mm_dbg ("Parsing PDU (%u)...", index);
+        mm_obj_dbg (log_object, "parsing PDU (%u)...", index);
     else
-        mm_dbg ("Parsing PDU...");
+        mm_obj_dbg (log_object, "parsing PDU...");
 
 #define PDU_SIZE_CHECK(required_size, check_descr_str)                 \
     if (pdu_len < required_size) {                                     \
@@ -414,12 +412,17 @@ mm_sms_part_3gpp_new_from_binary_pdu (guint index,
     if (smsc_addr_size_bytes > 0) {
         PDU_SIZE_CHECK (offset + smsc_addr_size_bytes, "cannot read SMSC address");
         /* SMSC may not be given in DELIVER PDUs */
-        mm_sms_part_take_smsc (sms_part,
-                               sms_decode_address (&pdu[1], 2 * (smsc_addr_size_bytes - 1)));
-        mm_dbg ("  SMSC address parsed: '%s'", mm_sms_part_get_smsc (sms_part));
+        address = sms_decode_address (&pdu[1], 2 * (smsc_addr_size_bytes - 1), error);
+        if (!address) {
+            g_prefix_error (error, "Couldn't read SMSC address: ");
+            mm_sms_part_free (sms_part);
+            return NULL;
+        }
+        mm_sms_part_take_smsc (sms_part, g_steal_pointer (&address));
+        mm_obj_dbg (log_object, "  SMSC address parsed: '%s'", mm_sms_part_get_smsc (sms_part));
         offset += smsc_addr_size_bytes;
     } else
-        mm_dbg ("  No SMSC address given");
+        mm_obj_dbg (log_object, "  no SMSC address given");
 
 
     /* ---------------------------------------------------------------------- */
@@ -429,15 +432,15 @@ mm_sms_part_3gpp_new_from_binary_pdu (guint index,
     pdu_type = (pdu[offset] & SMS_TP_MTI_MASK);
     switch (pdu_type) {
     case SMS_TP_MTI_SMS_DELIVER:
-        mm_dbg ("  Deliver type PDU detected");
+        mm_obj_dbg (log_object, "  deliver type PDU detected");
         mm_sms_part_set_pdu_type (sms_part, MM_SMS_PDU_TYPE_DELIVER);
         break;
     case SMS_TP_MTI_SMS_SUBMIT:
-        mm_dbg ("  Submit type PDU detected");
+        mm_obj_dbg (log_object, "  submit type PDU detected");
         mm_sms_part_set_pdu_type (sms_part, MM_SMS_PDU_TYPE_SUBMIT);
         break;
     case SMS_TP_MTI_SMS_STATUS_REPORT:
-        mm_dbg ("  Status report type PDU detected");
+        mm_obj_dbg (log_object, "  status report type PDU detected");
         mm_sms_part_set_pdu_type (sms_part, MM_SMS_PDU_TYPE_STATUS_REPORT);
         break;
     default:
@@ -470,7 +473,7 @@ mm_sms_part_3gpp_new_from_binary_pdu (guint index,
         pdu_type == SMS_TP_MTI_SMS_SUBMIT) {
         PDU_SIZE_CHECK (offset + 1, "cannot read message reference");
 
-        mm_dbg ("  message reference: %u", (guint)pdu[offset]);
+        mm_obj_dbg (log_object, "  message reference: %u", (guint)pdu[offset]);
         mm_sms_part_set_message_reference (sms_part, pdu[offset]);
         offset++;
     }
@@ -487,10 +490,14 @@ mm_sms_part_3gpp_new_from_binary_pdu (guint index,
     tp_addr_size_bytes = (tp_addr_size_digits + 1) >> 1;
 
     PDU_SIZE_CHECK (offset + tp_addr_size_bytes, "cannot read number");
-    mm_sms_part_take_number (sms_part,
-                             sms_decode_address (&pdu[offset],
-                                                 tp_addr_size_digits));
-    mm_dbg ("  Number parsed: '%s'", mm_sms_part_get_number (sms_part));
+    address = sms_decode_address (&pdu[offset], tp_addr_size_digits, error);
+    if (!address) {
+        g_prefix_error (error, "Couldn't read address: ");
+        mm_sms_part_free (sms_part);
+        return NULL;
+    }
+    mm_sms_part_take_number (sms_part, g_steal_pointer (&address));
+    mm_obj_dbg (log_object, "  number parsed: %s", mm_sms_part_get_number (sms_part));
     offset += (1 + tp_addr_size_bytes); /* +1 due to the Type of Address byte */
 
     /* ---------------------------------------------------------------------- */
@@ -526,20 +533,20 @@ mm_sms_part_3gpp_new_from_binary_pdu (guint index,
         if (validity_format) {
             switch (validity_format) {
             case 0x10:
-                mm_dbg ("  validity available, format relative");
+                mm_obj_dbg (log_object, "  validity available, format relative");
                 mm_sms_part_set_validity_relative (sms_part,
                                                    relative_to_validity (pdu[offset]));
                 offset++;
                 break;
             case 0x08:
                 /* TODO: support enhanced format; GSM 03.40 */
-                mm_dbg ("  validity available, format enhanced (not implemented)");
+                mm_obj_dbg (log_object, "  validity available, format enhanced (not implemented)");
                 /* 7 bytes for enhanced validity */
                 offset += 7;
                 break;
             case 0x18:
                 /* TODO: support absolute format; GSM 03.40 */
-                mm_dbg ("  validity available, format absolute (not implemented)");
+                mm_obj_dbg (log_object, "  validity available, format absolute (not implemented)");
                 /* 7 bytes for absolute validity */
                 offset += 7;
                 break;
@@ -569,7 +576,7 @@ mm_sms_part_3gpp_new_from_binary_pdu (guint index,
         offset += 7;
 
         /* ----- TP-STATUS (1 byte) ------ */
-        mm_dbg ("  delivery state: %u", (guint)pdu[offset]);
+        mm_obj_dbg (log_object, "  delivery state: %u", (guint)pdu[offset]);
         mm_sms_part_set_delivery_state (sms_part, pdu[offset]);
         offset++;
 
@@ -594,7 +601,7 @@ mm_sms_part_3gpp_new_from_binary_pdu (guint index,
 
     if (tp_pid_offset > 0) {
         PDU_SIZE_CHECK (tp_pid_offset + 1, "cannot read TP-PID");
-        mm_dbg ("  PID: %u", (guint)pdu[tp_pid_offset]);
+        mm_obj_dbg (log_object, "  PID: %u", (guint)pdu[tp_pid_offset]);
     }
 
     /* Grab user data encoding and message class */
@@ -605,17 +612,20 @@ mm_sms_part_3gpp_new_from_binary_pdu (guint index,
         user_data_encoding = sms_encoding_type (pdu[tp_dcs_offset]);
         switch (user_data_encoding) {
         case MM_SMS_ENCODING_GSM7:
-            mm_dbg ("  user data encoding is GSM7");
+            mm_obj_dbg (log_object, "  user data encoding is GSM7");
             break;
         case MM_SMS_ENCODING_UCS2:
-            mm_dbg ("  user data encoding is UCS2");
+            mm_obj_dbg (log_object, "  user data encoding is UCS2");
             break;
         case MM_SMS_ENCODING_8BIT:
-            mm_dbg ("  user data encoding is 8bit");
+            mm_obj_dbg (log_object, "  user data encoding is 8bit");
+            break;
+        case MM_SMS_ENCODING_UNKNOWN:
+            mm_obj_dbg (log_object, "  user data encoding is unknown");
             break;
         default:
-            mm_dbg ("  user data encoding is unknown");
-            break;
+            g_assert_not_reached ();
+
         }
         mm_sms_part_set_encoding (sms_part, user_data_encoding);
 
@@ -633,13 +643,13 @@ mm_sms_part_3gpp_new_from_binary_pdu (guint index,
 
         PDU_SIZE_CHECK (tp_user_data_len_offset + 1, "cannot read TP-UDL");
         tp_user_data_size_elements = pdu[tp_user_data_len_offset];
-        mm_dbg ("  user data length: %u elements", tp_user_data_size_elements);
+        mm_obj_dbg (log_object, "  user data length: %u elements", tp_user_data_size_elements);
 
         if (user_data_encoding == MM_SMS_ENCODING_GSM7)
             tp_user_data_size_bytes = (7 * (tp_user_data_size_elements + 1 )) / 8;
         else
             tp_user_data_size_bytes = tp_user_data_size_elements;
-        mm_dbg ("  user data length: %u bytes", tp_user_data_size_bytes);
+        mm_obj_dbg (log_object, "  user data length: %u bytes", tp_user_data_size_bytes);
 
         tp_user_data_offset = tp_user_data_len_offset + 1;
         PDU_SIZE_CHECK (tp_user_data_offset + tp_user_data_size_bytes, "cannot read TP-UD");
@@ -688,6 +698,8 @@ mm_sms_part_3gpp_new_from_binary_pdu (guint index,
                     mm_sms_part_set_concat_max (sms_part,pdu[offset + 2]);
                     mm_sms_part_set_concat_sequence (sms_part, pdu[offset + 3]);
                     break;
+                default:
+                    break;
                 }
 
                 offset += ie_len;
@@ -713,21 +725,31 @@ mm_sms_part_3gpp_new_from_binary_pdu (guint index,
         switch (user_data_encoding) {
         case MM_SMS_ENCODING_GSM7:
         case MM_SMS_ENCODING_UCS2:
-            /* Otherwise if it's 7-bit or UCS2 we can decode it */
-            mm_dbg ("Decoding SMS text with '%u' elements", tp_user_data_size_elements);
-            mm_sms_part_take_text (sms_part,
-                                   sms_decode_text (&pdu[tp_user_data_offset],
-                                                    tp_user_data_size_elements,
-                                                    user_data_encoding,
-                                                    bit_offset));
-            g_warn_if_fail (mm_sms_part_get_text (sms_part) != NULL);
-            break;
+            {
+                gchar *text;
 
+                /* Otherwise if it's 7-bit or UCS2 we can decode it */
+                mm_obj_dbg (log_object, "decoding SMS text with %u elements", tp_user_data_size_elements);
+                text = sms_decode_text (&pdu[tp_user_data_offset],
+                                        tp_user_data_size_elements,
+                                        user_data_encoding,
+                                        bit_offset,
+                                        log_object,
+                                        error);
+                if (!text) {
+                    mm_sms_part_free (sms_part);
+                    return NULL;
+                }
+                mm_sms_part_take_text (sms_part, text);
+                break;
+            }
+        case MM_SMS_ENCODING_8BIT:
+        case MM_SMS_ENCODING_UNKNOWN:
         default:
             {
                 GByteArray *raw;
 
-                mm_dbg ("Skipping SMS text: Unknown encoding (0x%02X)", user_data_encoding);
+                mm_obj_dbg (log_object, "skipping SMS text: unknown encoding (0x%02X)", user_data_encoding);
 
                 PDU_SIZE_CHECK (tp_user_data_offset + tp_user_data_size_bytes, "cannot read user data");
 
@@ -751,7 +773,7 @@ mm_sms_part_3gpp_new_from_binary_pdu (guint index,
  * @address: the phone number to encode
  * @buf: the buffer to encode @address in
  * @buflen: the size  of @buf
- * @is_smsc: if %TRUE encode size as number of octets of address infromation,
+ * @is_smsc: if %TRUE encode size as number of octets of address information,
  *   otherwise if %FALSE encode size as number of digits of @address
  *
  * Returns: the size in bytes of the data added to @buf
@@ -805,12 +827,14 @@ guint8 *
 mm_sms_part_3gpp_get_submit_pdu (MMSmsPart *part,
                                  guint *out_pdulen,
                                  guint *out_msgstart,
+                                 gpointer log_object,
                                  GError **error)
 {
     guint8 *pdu;
     guint len, offset = 0;
     guint shift = 0;
     guint8 *udl_ptr;
+    MMSmsEncoding encoding;
 
     g_return_val_if_fail (mm_sms_part_get_number (part) != NULL, NULL);
     g_return_val_if_fail (mm_sms_part_get_text (part) != NULL || mm_sms_part_get_data (part) != NULL, NULL);
@@ -824,13 +848,13 @@ mm_sms_part_3gpp_get_submit_pdu (MMSmsPart *part,
         return NULL;
     }
 
-    mm_dbg ("Creating PDU for part...");
+    mm_obj_dbg (log_object, "creating PDU for part...");
 
     /* Build up the PDU */
     pdu = g_malloc0 (PDU_SIZE);
 
     if (mm_sms_part_get_smsc (part)) {
-        mm_dbg ("  adding SMSC to PDU...");
+        mm_obj_dbg (log_object, "  adding SMSC to PDU...");
         len = mm_sms_part_3gpp_encode_address (mm_sms_part_get_smsc (part), pdu, PDU_SIZE, TRUE);
         if (len == 0) {
             g_set_error (error,
@@ -853,13 +877,13 @@ mm_sms_part_3gpp_get_submit_pdu (MMSmsPart *part,
 
     /* TP-VP present; format RELATIVE */
     if (mm_sms_part_get_validity_relative (part) > 0) {
-        mm_dbg ("  adding validity to PDU...");
+        mm_obj_dbg (log_object, "  adding validity to PDU...");
         pdu[offset] |= 0x10;
     }
 
     /* Concatenation sequence only found in multipart SMS */
     if (mm_sms_part_get_concat_sequence (part)) {
-        mm_dbg ("  adding UDHI to PDU...");
+        mm_obj_dbg (log_object, "  adding UDHI to PDU...");
         pdu[offset] |= 0x40; /* UDHI */
     }
 
@@ -868,7 +892,7 @@ mm_sms_part_3gpp_get_submit_pdu (MMSmsPart *part,
     if (mm_sms_part_get_delivery_report_request (part) &&
         (!mm_sms_part_get_concat_sequence (part) ||
          mm_sms_part_get_concat_max (part) == mm_sms_part_get_concat_sequence (part))) {
-        mm_dbg ("  requesting delivery report...");
+        mm_obj_dbg (log_object, "  requesting delivery report...");
         pdu[offset] |= 0x20;
     }
 
@@ -900,22 +924,26 @@ mm_sms_part_3gpp_get_submit_pdu (MMSmsPart *part,
     pdu[offset] = 0x00;
 
     if (mm_sms_part_get_class (part) >= 0 && mm_sms_part_get_class (part) <= 3) {
-        mm_dbg ("  using class %d...", mm_sms_part_get_class (part));
+        mm_obj_dbg (log_object, "  using class %d...", mm_sms_part_get_class (part));
         pdu[offset] |= SMS_DCS_CLASS_VALID;
         pdu[offset] |= mm_sms_part_get_class (part);
     }
 
-    switch (mm_sms_part_get_encoding (part)) {
+    encoding = mm_sms_part_get_encoding (part);
+
+    switch (encoding) {
     case MM_SMS_ENCODING_UCS2:
-        mm_dbg ("  using UCS2 encoding...");
+        mm_obj_dbg (log_object, "  using UCS2 encoding...");
         pdu[offset] |= SMS_DCS_CODING_UCS2;
         break;
     case MM_SMS_ENCODING_GSM7:
-        mm_dbg ("  using GSM7 encoding...");
+        mm_obj_dbg (log_object, "  using GSM7 encoding...");
         pdu[offset] |= SMS_DCS_CODING_DEFAULT;  /* GSM */
         break;
+    case MM_SMS_ENCODING_8BIT:
+    case MM_SMS_ENCODING_UNKNOWN:
     default:
-        mm_dbg ("  using 8bit encoding...");
+        mm_obj_dbg (log_object, "  using 8bit encoding...");
         pdu[offset] |= SMS_DCS_CODING_8BIT;
         break;
     }
@@ -934,7 +962,7 @@ mm_sms_part_3gpp_get_submit_pdu (MMSmsPart *part,
 
     /* Build UDH */
     if (mm_sms_part_get_concat_sequence (part)) {
-        mm_dbg ("  adding UDH header in PDU... (reference: %u, max: %u, sequence: %u)",
+        mm_obj_dbg (log_object, "  adding UDH header in PDU... (reference: %u, max: %u, sequence: %u)",
                 mm_sms_part_get_concat_reference (part),
                 mm_sms_part_get_concat_max (part),
                 mm_sms_part_get_concat_sequence (part));
@@ -956,13 +984,16 @@ mm_sms_part_3gpp_get_submit_pdu (MMSmsPart *part,
         shift = 1;
     }
 
-    if (mm_sms_part_get_encoding (part) == MM_SMS_ENCODING_GSM7) {
-        guint8 *unpacked, *packed;
-        guint32 unlen = 0, packlen = 0;
+    if (encoding == MM_SMS_ENCODING_GSM7) {
+        g_autoptr(GByteArray)  unpacked = NULL;
+        g_autofree guint8     *packed = NULL;
+        guint32                packlen = 0;
 
-        unpacked = mm_charset_utf8_to_unpacked_gsm (mm_sms_part_get_text (part), &unlen);
-        if (!unpacked || unlen == 0) {
-            g_free (unpacked);
+        unpacked = mm_modem_charset_bytearray_from_utf8 (mm_sms_part_get_text (part), MM_MODEM_CHARSET_GSM, FALSE, error);
+        if (!unpacked)
+            goto error;
+
+        if (unpacked->len == 0) {
             g_set_error_literal (error,
                                  MM_MESSAGE_ERROR,
                                  MM_MESSAGE_ERROR_INVALID_PDU_PARAMETER,
@@ -973,15 +1004,13 @@ mm_sms_part_3gpp_get_submit_pdu (MMSmsPart *part,
         /* Set real data length, in septets
          * If we had UDH, add 7 septets
          */
-        *udl_ptr = mm_sms_part_get_concat_sequence (part) ? (7 + unlen) : unlen;
-        mm_dbg ("  user data length is '%u' septets (%s UDH)",
-                *udl_ptr,
-                mm_sms_part_get_concat_sequence (part) ? "with" : "without");
+        *udl_ptr = mm_sms_part_get_concat_sequence (part) ? (7 + unpacked->len) : unpacked->len;
+        mm_obj_dbg (log_object, "  user data length is %u septets (%s UDH)",
+                    *udl_ptr,
+                    mm_sms_part_get_concat_sequence (part) ? "with" : "without");
 
-        packed = mm_charset_gsm_pack (unpacked, unlen, shift, &packlen);
-        g_free (unpacked);
+        packed = mm_charset_gsm_pack (unpacked->data, unpacked->len, shift, &packlen);
         if (!packed || packlen == 0) {
-            g_free (packed);
             g_set_error_literal (error,
                                  MM_MESSAGE_ERROR,
                                  MM_MESSAGE_ERROR_INVALID_PDU_PARAMETER,
@@ -990,19 +1019,19 @@ mm_sms_part_3gpp_get_submit_pdu (MMSmsPart *part,
         }
 
         memcpy (&pdu[offset], packed, packlen);
-        g_free (packed);
         offset += packlen;
-    } else if (mm_sms_part_get_encoding (part) == MM_SMS_ENCODING_UCS2) {
-        GByteArray *array;
+    } else if (encoding == MM_SMS_ENCODING_UCS2) {
+        g_autoptr(GByteArray) array = NULL;
+        g_autoptr(GError)     inner_error = NULL;
 
-        /* Try to guess a good value for the array */
-        array = g_byte_array_sized_new (strlen (mm_sms_part_get_text (part)) * 2);
-        if (!mm_modem_charset_byte_array_append (array, mm_sms_part_get_text (part), FALSE, MM_MODEM_CHARSET_UCS2)) {
-            g_byte_array_free (array, TRUE);
-            g_set_error_literal (error,
-                                 MM_MESSAGE_ERROR,
-                                 MM_MESSAGE_ERROR_INVALID_PDU_PARAMETER,
-                                 "Failed to convert message text to UCS2");
+        /* Always assume UTF-16 instead of UCS-2! */
+        array = mm_modem_charset_bytearray_from_utf8 (mm_sms_part_get_text (part), MM_MODEM_CHARSET_UTF16, FALSE, &inner_error);
+        if (!array) {
+            g_set_error (error,
+                         MM_MESSAGE_ERROR,
+                         MM_MESSAGE_ERROR_INVALID_PDU_PARAMETER,
+                         "Failed to convert message text to UTF-16: %s",
+                         inner_error->message);
             goto error;
         }
 
@@ -1010,13 +1039,12 @@ mm_sms_part_3gpp_get_submit_pdu (MMSmsPart *part,
          * If we had UDH, add 6 octets
          */
         *udl_ptr = mm_sms_part_get_concat_sequence (part) ? (6 + array->len) : array->len;
-        mm_dbg ("  user data length is '%u' octets (%s UDH)",
-                *udl_ptr,
-                mm_sms_part_get_concat_sequence (part) ? "with" : "without");
+        mm_obj_dbg (log_object, "  user data length is %u octets (%s UDH)",
+                    *udl_ptr,
+                    mm_sms_part_get_concat_sequence (part) ? "with" : "without");
 
         memcpy (&pdu[offset], array->data, array->len);
         offset += array->len;
-        g_byte_array_free (array, TRUE);
     } else if (mm_sms_part_get_encoding (part) == MM_SMS_ENCODING_8BIT) {
         const GByteArray *data;
 
@@ -1026,7 +1054,7 @@ mm_sms_part_3gpp_get_submit_pdu (MMSmsPart *part,
          * If we had UDH, add 6 octets
          */
         *udl_ptr = mm_sms_part_get_concat_sequence (part) ? (6 + data->len) : data->len;
-        mm_dbg ("  binary user data length is '%u' octets (%s UDH)",
+        mm_obj_dbg (log_object, "  binary user data length is %u octets (%s UDH)",
                 *udl_ptr,
                 mm_sms_part_get_concat_sequence (part) ? "with" : "without");
 
@@ -1044,21 +1072,109 @@ error:
     return NULL;
 }
 
-gchar **
-mm_sms_part_3gpp_util_split_text (const gchar *text,
-                                  MMSmsEncoding *encoding)
+static gchar **
+util_split_text_gsm7 (const gchar *text,
+                      gsize        text_len,
+                      gpointer     log_object)
 {
     gchar **out;
-    guint n_chunks;
-    guint i;
-    guint j;
-    gsize in_len;
+    guint   n_chunks;
+    guint   i;
+    guint   j;
 
+    /* No splitting needed? */
+    if (text_len <= 160) {
+        out = g_new0 (gchar *, 2);
+        out[0] = g_strdup (text);
+        return out;
+    }
+
+    /* Compute number of chunks needed */
+    n_chunks = text_len / 153;
+    if (text_len % 153 != 0)
+        n_chunks++;
+
+    /* Fill in all chunks */
+    out = g_new0 (gchar *, n_chunks + 1);
+    for (i = 0, j = 0; i < n_chunks; i++, j += 153)
+        out[i] = g_strndup (&text[j], 153);
+
+    return out;
+}
+
+static gchar **
+util_split_text_utf16_or_ucs2 (const gchar *text,
+                               gsize        text_len,
+                               gpointer     log_object)
+{
+    g_autoptr(GPtrArray)  chunks = NULL;
+    const gchar          *walker;
+    const gchar          *chunk_start;
+    glong                 encoded_chunk_length;
+    glong                 total_encoded_chunk_length;
+
+    chunks = g_ptr_array_new_with_free_func ((GDestroyNotify)g_free);
+
+    walker = text;
+    chunk_start = text;
+    encoded_chunk_length = 0;
+    total_encoded_chunk_length = 0;
+    while (walker && *walker) {
+        g_autofree gunichar2 *unichar2 = NULL;
+        glong                 unichar2_written = 0;
+        glong                 unichar2_written_bytes = 0;
+        gunichar              single;
+
+        single = g_utf8_get_char (walker);
+        unichar2 = g_ucs4_to_utf16 (&single, 1, NULL, &unichar2_written, NULL);
+        g_assert (unichar2_written > 0);
+
+        /* When splitting for UCS-2 encoding, only one single unichar2 will be
+         * written, because all codepoints represented in UCS2 fit in the BMP.
+         * When splitting for UTF-16, though, we may end up writing one or two
+         * unichar2 (without or with surrogate pairs), because UTF-16 covers the
+         * whole Unicode spectrum. */
+        unichar2_written_bytes = (unichar2_written * sizeof (gunichar2));
+        if ((encoded_chunk_length + unichar2_written_bytes) > 134) {
+            g_ptr_array_add (chunks, g_strndup (chunk_start, walker - chunk_start));
+            chunk_start = walker;
+            encoded_chunk_length = unichar2_written_bytes;
+        } else
+            encoded_chunk_length += unichar2_written_bytes;
+
+        total_encoded_chunk_length += unichar2_written_bytes;
+        walker = g_utf8_next_char (walker);
+    }
+
+    /* We have split the original string in chunks, where each chunk
+     * does not require more than 134 bytes when encoded in UTF-16.
+     * As a special case now, we consider the case that no splitting
+     * is necessary, i.e. if the total amount of bytes after encoding
+     * in UTF-16 is less or equal than 140. */
+    if (total_encoded_chunk_length <= 140) {
+        gchar **out;
+
+        out = g_new0 (gchar *, 2);
+        out[0] = g_strdup (text);
+        return out;
+    }
+
+    /* Otherwise, we do need the splitted chunks. Add the last one
+     * with contents plus the last trailing NULL */
+    g_ptr_array_add (chunks, g_strndup (chunk_start, walker - chunk_start));
+    g_ptr_array_add (chunks, NULL);
+
+    return (gchar **) g_ptr_array_free (g_steal_pointer (&chunks), FALSE);
+}
+
+gchar **
+mm_sms_part_3gpp_util_split_text (const gchar   *text,
+                                  MMSmsEncoding *encoding,
+                                  gpointer       log_object)
+{
     if (!text)
         return NULL;
 
-    in_len = strlen (text);
-
     /* Some info about the rules for splitting.
      *
      * The User Data can be up to 140 bytes in the SMS part:
@@ -1073,78 +1189,25 @@ mm_sms_part_3gpp_util_split_text (const gchar *text,
      *      134 * 8 = 1072; 1072/7=153.14
      *  2) If we're using UCS2 encoding, we can pack up to 70 characters in
      *     140 bytes (each with 2 bytes), or up to 67 characters in 134 bytes.
+     *  3) If we're using UTF-16 encoding (instead of UCS2), the amount of
+     *     characters we can pack is variable, depends on how the characters
+     *     are encoded in UTF-16 (e.g. if there are characters out of the BMP
+     *     we'll need surrogate pairs and a single character will need 4 bytes
+     *     instead of 2).
      *
      * This method does the split of the input string into N strings, so that
      * each of the strings can be placed in a SMS part.
      */
 
     /* Check if we can do GSM encoding */
-    if (!mm_charset_can_convert_to (text, MM_MODEM_CHARSET_GSM)) {
-        /* If cannot do it in GSM encoding, do it in UCS-2 */
-        GByteArray *array;
-
-        *encoding = MM_SMS_ENCODING_UCS2;
-
-        /* Guess more or less the size of the output array to avoid multiple
-         * allocations */
-        array = g_byte_array_sized_new (in_len * 2);
-        if (!mm_modem_charset_byte_array_append (array,
-                                                 text,
-                                                 FALSE,
-                                                 MM_MODEM_CHARSET_UCS2)) {
-            g_byte_array_unref (array);
-            return NULL;
-        }
-
-        /* Our bytearray has it in UCS-2 now.
-         * UCS-2 is a fixed-size encoding, which means that the text has exactly
-         * 2 bytes for each unicode point. We can now split this array into
-         * chunks of 67 UCS-2 characters (134 bytes).
-         *
-         * Note that UCS-2 covers unicode points between U+0000 and U+FFFF, which
-         * means that there is no direct relationship between the size of the
-         * input text in UTF-8 and the size of the text in UCS-2. A 3-byte UTF-8
-         * encoded character will still be represented with 2 bytes in UCS-2.
-         */
-        if (array->len <= 140) {
-            out = g_new (gchar *, 2);
-            out[0] = g_strdup (text);
-            out[1] = NULL;
-        } else {
-            n_chunks = array->len / 134;
-            if (array->len % 134 != 0)
-                n_chunks++;
-
-            out = g_new0 (gchar *, n_chunks + 1);
-            for (i = 0, j = 0; i < n_chunks; i++, j += 134) {
-                out[i] = sms_decode_text (&array->data[j],
-                                          MIN (array->len - j, 134),
-                                          MM_SMS_ENCODING_UCS2,
-                                          0);
-            }
-        }
-        g_byte_array_unref (array);
-    } else {
-        /* Do it with GSM encoding */
+    if (mm_charset_can_convert_to (text, MM_MODEM_CHARSET_GSM)) {
         *encoding = MM_SMS_ENCODING_GSM7;
-
-        if (in_len <= 160) {
-            out = g_new (gchar *, 2);
-            out[0] = g_strdup (text);
-            out[1] = NULL;
-        } else {
-            n_chunks = in_len / 153;
-            if (in_len % 153 != 0)
-                n_chunks++;
-
-            out = g_new0 (gchar *, n_chunks + 1);
-            for (i = 0, j = 0; i < n_chunks; i++, j += 153) {
-                out[i] = g_strndup (&text[j], 153);
-            }
-        }
+        return util_split_text_gsm7 (text, strlen (text), log_object);
     }
 
-    return out;
+    /* Otherwise fallback to report UCS-2 and split supporting UTF-16 */
+    *encoding = MM_SMS_ENCODING_UCS2;
+    return util_split_text_utf16_or_ucs2 (text, strlen (text), log_object);
 }
 
 GByteArray **
